/*
	Write by xempo 10-2014
	Free under LGPL
*/

#include <qclipboard.h>
#include <qcursor.h>
#include <qpainter.h>
#include "filesan.h"
#include "pluzone.h"
#include "simpleimage.h"
#include "inputdlg.h"
#include "fspcon.h"
#include "../rtdata/en/target/filesor.h"
#include "../rtdata/en/target/simage.h"

extern QMetaData popPluzoneSlot; // searchmodify.cpp

const char simpleImage[]="SimpleImage",  AlphChs[]=", Alpha";

QPtrList <SimpleImage> grapList;

SimpleImage::SimpleImage(QWidget *par) : QMainWindow(par, simpleImage, WDestructiveClose|WType_TopLevel),
	fs((Filesan *)par)
{
fs->busiedCursor();
grapList.append(this);
}

SimpleImage::~SimpleImage(){}

void SimpleImage::countWH(int &w1, int &h1)
{
	int w=img->width(), h=img->height(),
	wlmt=screenWidth*9/10,
	hlmt=screenHeight*17/20;
	w1=w;	h1=h;
	if(h>hlmt){ w1=w*hlmt/h; h1=hlmt; }
	if(w1>wlmt){ w1=wlmt;  h1=h*wlmt/w; }
	P=w1*100/w;
}

void SimpleImage::exe(const char *fnam)
{
    const char **_simage_=charsFromFSP(7);
    int w1, h1;
    tb=new QToolBar(simpleImage, this, Qt::DockBottom);
    tb->setMovingEnabled(false);
    mdfAct=addToolAction(this, tb, simpleImageImg, editChs, NULL, SLOT(pop()), ALT+Key_E, false, false);
    tb->addSeparator();
    moreSizeAct=addToolAction(this, tb, moreSizeImg, moreSizeChs, NULL, SLOT(moreSize()), CTRL+Key_M, false, false);
    lessSizeAct=addToolAction(this, tb, lessSizeImg, lessSizeChs, NULL, SLOT(lessSize()), CTRL+Key_L, false, false);
    normalSizeAct=addToolAction(this, tb, normalSizeImg, normalSizeChs, NULL, SLOT(normalSize()), CTRL+Key_N, false, false);
    tb->addSeparator();
    saveAct=addToolAction(this, tb, saveImg, saveChs, NULL, SLOT(save()), CTRL+Key_S, false, false);

    sbar=new StatusPanel(tb, sbar_Grap);
    tb->setStretchableWidget(sbar);
    lab=new QLabel(tb);
    helpAct=addToolAction(this, tb, faqImg, helpChs, NULL, slot_help, Key_F1, false, false);

    fn=fnam;
    setCaption( strCodec->toUnicode( fnam ) );
    img=new QImage(caption(), NULL);
    if( img->isNull() ){
	char buf[2400];
	sprintf(buf, notImageFile, fnam);
	lab->setText(QString::fromUtf8(buf));
	w1=0;	h1=0;
	}
    else {
	countWH(w1, h1);
	setIcon(*img);
	status();
	}
    resize( w1<400 ? 480 : w1, h1<360 ? 360 : h1 );
    show();
    editMenu=new ActMenu(this);
    static const int depthVal[3]={ 1, 8, 32 };
    static const char DepthSlot[]=SLOT(cvtDepth(int));
    copyId=editMenu->insertMenu(copyImg, fcopy, this, SLOT(copy()), CTRL+Key_C);
    pasteId=editMenu->insertMenu(pasteImg, pasteChs, this, SLOT(paste()), CTRL+Key_V);
    editMenu->insertSeparator();
    flip1Id=editMenu->insertItem( QString::fromUtf8(hFlipChs), this, SLOT(hFlip()) );
    flip2Id=editMenu->insertItem( QString::fromUtf8(vFlipChs), this, SLOT(vFlip()) );
    flip3Id=editMenu->insertItem( QString::fromUtf8(Flip180Chs), this, SLOT(Flip180()) );
    editMenu->insertSeparator();
    scaleId=editMenu->insertItem( QString::fromUtf8(scaleChs), this, SLOT(scale()) );
    editMenu->insertSeparator();
    for(int n=0;n<3;n++){
	depthId[n]=editMenu->insertItem( QString::fromUtf8(_simage_[n]), this, DepthSlot );
	editMenu->setItemParameter(depthId[n], depthVal[n] );
	}
free((void *)_simage_[0]);
free((void *)_simage_);
fs->normalCursor();
}

void SimpleImage::status()
{
char buff[80];
sprintf(buff, "%dx%d%s  %d%c ", img->width(), img->height(), img->hasAlphaBuffer() ? AlphChs : "", P, '%');
lab->setText(QString::fromUtf8(buff));
}

void SimpleImage::imageMirror( bool h, bool v)
{
*img=img->mirror(h,v);
repaint();
status();
}

void SimpleImage::imageCopy()
{
QApplication::clipboard()->setImage( *img );
// mainWidget->MainWdgState(this);
}

void SimpleImage::imagePaste()
{
int w, h;
*img=QApplication::clipboard()->image();
// P=100;
countWH(w, h);
repaint();
status();
}

bool SimpleImage::event( QEvent *e )
{
switch(e->type()){
case QEvent::Paint : if(img->isNull()==false){
	QPainter painter(this);
	int w=img->width()*P/100, h=img->height()*P/100;
	painter.drawPixmap(1, 1, P==100 ? *img :
		( w>400||h>300 ? img->scale(w, h) : img->smoothScale(w, h) ));
	}
	return true;
case QEvent::Close :
	menuSImage.add(fn);
	free((void *)fn);
	grapList.remove(this);
	break;
case QEvent::ShowNormal :
	tms=0;
	tmId=startTimer(60);
	return true;
case QEvent::KeyPress :
	switch(((QKeyEvent *)e)->key()){
	case Key_F5 : if(grapList.count()>0)showTop( grapList.at(1) );
	}
	break;
case QEvent::Timer :
	if(++tms>5){
		killTimer(tmId);
		raise();	setActiveWindow();
		}
	return true;
case QEvent::WindowDeactivate :
case QEvent::WindowActivate : // first than WindowActivate //先失去焦点，后再另设焦点
	if(grapList.count()>1 && grapList.remove(this) )grapList.prepend(this);
	return true;
}
return QMainWindow::event( e );
}

QMetaObject *SimpleImage::metaObj = 0;
static QMetaObjectCleanUp cleanUp_SimpleImage( simpleImage, &SimpleImage::staticMetaObject );
QMetaObject *SimpleImage::staticMetaObject()
{
if( ! metaObj ){
 static const QMetaData btn_tbl[]={
	{ "scale()", &slot_0_0, QMetaData::Private },
	{ "cvtDepth(int)", &slot_int, QMetaData::Private },
	{ "Flip180()", &slot_0_0, QMetaData::Private },
	{ "hFlip()", &slot_0_0, QMetaData::Private },
	{ "vFlip()", &slot_0_0, QMetaData::Private },
	{ "copy()", &slot_0_0, QMetaData::Private },
	{ "paste()", &slot_0_0, QMetaData::Private },
	{ "pop()", &slot_0_0, QMetaData::Private },
	helpSlot,
	{ "save()", &slot_0_0, QMetaData::Private },
	closeAllWdgSlot,
	closeAllSlot,
	recentSlot,
	popActSlot,
	{ "moreSize()", &slot_0_0, QMetaData::Private },
	{ "lessSize()", &slot_0_0, QMetaData::Private },
	{ "normalSize()", &slot_0_0, QMetaData::Private },
	clearBrwSlot,
	popPluzoneSlot,
	menuOpenSlot
	};
 metaObj= QMetaObject::new_metaobject(simpleImage, QMainWindow::staticMetaObject(), btn_tbl, 20,
	0, 0,
#ifndef QT_NO_PROPERTIES
	0, 0,
	0, 0,
#endif // QT_NO_PROPERTIES
	0, 0 );
cleanUp_SimpleImage.setMetaObject(metaObj);
}
return metaObj;
}

#ifndef QT_NO_PROPERTIES
bool SimpleImage::qt_static_property( QObject* , int , int , QVariant* ){ return false; }
#endif // QT_NO_PROPERTIES

bool SimpleImage::qt_invoke(int _id, QUObject *_o)
{
static const char fmt[]="%d";
int msg, i, sw=_id - staticMetaObject()->slotOffset();
switch(sw){
case 0 : {
	int x1, y1, oldx, oldy;
	static FastBlock x_lst(CombList), y_lst(CombList);
	QString retS;
	char xbu[36], ybu[36];
	oldx=img->width();
	oldy=img->height();
	sprintf(xbu, fmt, oldx );
	sprintf(ybu, fmt, oldy );
	InputDlg wdg(this, 3, pms_simage/* mode */);
	msg=wdg.exe(retS, NULL, &x_lst, NULL, &y_lst);
	if( (msg & ~(64|128|256))!=0 ) break;
	pms_simage=msg;
	x1=atoi(retS);
	if(msg&64){ // 按比例
		char *pt;
		float cvtY= (float)(x1 * oldy) / (float)oldx;
		sprintf(ybu, "%f", cvtY);
		y1=(int)cvtY;
		pt=strchr(ybu, '.');
		if(pt[1]>'4')y1++;
		replchs.sprintf(fmt, y1);
	}
	else y1=atoi(replchs);
	*img=(msg&128) ? (msg&256 ? img->smoothScale(x1, y1) : img->scale(x1,y1)) : img->copy(0,0,x1,y1);
	repaint();
	status();
	}
	break;
case 1 : *img=img->convertDepth( _o[1].payload.i );
	repaint();
	status();
	break;
case 2 : imageMirror(true,false);
	break;
case 3 : imageMirror(false, true);
	break;
case 4 : imageMirror(true, true);
	break;
case 5 : imageCopy();
	break;
case 6 : imagePaste();
	setIcon(*img);
	break;
case 7 : bool b;
	b=!img->isNull();
	editMenu->setItemEnabled(copyId, b);
	editMenu->setItemEnabled(flip1Id, b);
	editMenu->setItemEnabled(flip2Id, b);
	editMenu->setItemEnabled(flip3Id, b);
	editMenu->setItemEnabled(scaleId, b);
	editMenu->setItemEnabled(pasteId, !QApplication::clipboard()->image().isNull());
	for(i=0; i<3; i++)
		editMenu->setItemEnabled(depthId[i], b);
	editMenu->exec(QCursor::pos());
	break;
case 8 : fsplusLink(this, 8, 2);
	break;
case 9 : {
	static char typeBuf[20]="PNG", qualityBuf[12]="0";
	static int qual;
	char buf[20];
	FastBlock x_lst, y_lst;
	QString retS;
	InputDlg wdg(this, 5, qual);
	for(i=20; i>-1; i--){
		sprintf(buf, fmt, i);
		y_lst.add(buf);
	}
	for(i=0; i<QImage::outputFormats().count(); i++)x_lst.add(QImage::outputFormats().at(i));
	msg=wdg.exe(retS, typeBuf, &x_lst, qualityBuf, &y_lst);
	if( (msg&7)!=0/* btn Idx, first-btn */) break;
	qual=msg&64;
	scopy(typeBuf, retS);	scopy(qualityBuf, msg&64 ? replchs : _Spc);
	if(strcmp(typeBuf,"JPG")==0)retS="JPEG";
	img->save( caption(), retS, msg&64 ? atoi(qualityBuf)*5 : -1 );
	}
	break;
case 10 : QApplication::postEvent(qApp, new QCloseEvent );
	break;
case 11 : appPost(65008, this, fs);
	break;
case 12 :
case 13 : showTop(grapList.at( sw!=12 ? _o[1].payload.i : 1 ));
	break;
case 14 : P=P*110/100;
	goto _paint_;
case 15 : P=P*90/100;
	goto _paint_;
case 16 : P=100;
_paint_:	repaint();
	status();
	break;
case 17 : menuSImage.resetMem(NULL);
	break;
case 18 : showTop(fs);
	break;
case 19 : {
	//busiedCursor();
	SimpleImage *gse=new SimpleImage(fs);
	gse->exe(strdup(menuSImage.lstIndex(_o[1].payload.i)));
	//normalCursor();
	}
	break;
default : return QMainWindow::qt_invoke(_id, _o);
}
return true;
}
